unit MainF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ToolWin, ExtCtrls, StdCtrls, Menus, {$IFNDEF VER100}ImgList, {$ENDIF}QLocal,
  QFormLocal, QLocalSource, QLocalUserSource;

type
  TfmMain = class(TForm)
    paTreeView: TPanel;
    sbSample: TStatusBar;
    tbSample: TToolBar;
    tvSample: TTreeView;
    paTreeViewCaption: TPanel;
    paClient: TPanel;
    paSampleControls: TPanel;
    spVertical: TSplitter;
    laSampleCombo: TLabel;
    cbSample: TComboBox;
    laSampleListBox: TLabel;
    lbSample: TListBox;
    rgSample: TRadioGroup;
    chSample1: TCheckBox;
    chSample2: TCheckBox;
    chSample3: TCheckBox;
    buSample1: TButton;
    buSample2: TButton;
    MainMenu: TMainMenu;
    miFile: TMenuItem;
    miExit: TMenuItem;
    miHelp: TMenuItem;
    miAbout: TMenuItem;
    pcSample: TPageControl;
    tsPage1: TTabSheet;
    tsPage2: TTabSheet;
    meSample: TMemo;
    stSample: TStaticText;
    miLanguage: TMenuItem;
    tbuLanguage: TToolButton;
    Images: TImageList;
    pmLanguages: TPopupMenu;
    tbuAbout: TToolButton;
    tsPage3: TTabSheet;
    lvSample: TListView;
    Localizer: TQFormLocalizer;
    LangSource: TQUserLanguageSource;
    procedure miAboutClick(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pmLanguagesPopup(Sender: TObject);
    procedure LocalizerLanguageChanged(Sender: TObject);
    procedure LangSourceLanguageChanged(Sender: TObject);
    procedure LocalizerLanguageChanging(Sender: TObject;
      LanguageIndex: Integer; var AllowChange: Boolean);
    procedure LangSourceLanguageOpen(Sender: TObject);
    procedure LangSourceLanguageClose(Sender: TObject);
    procedure LangSourceClear(Sender: TObject);
    procedure LangSourceClearAll(Sender: TObject);
    procedure LangSourceLoadString(Sender: TObject;
      const StringName: String; var Value: String);
    procedure LangSourceLoadStrings(Sender: TObject; Strings: TStrings);
  private
    FComboIndex: integer;
    FStrings: TStringList;
    procedure LanguageItemClick(Sender: TObject);
  end;

var
  fmMain: TfmMain;

implementation

uses AboutF, ShellApi, QLocalUtils;

resourcestring
  sAppTitle = 'EMS Advansed Localizer demo application';

{$R *.DFM}

procedure TfmMain.miAboutClick(Sender: TObject);
begin
  fmAbout := TfmAbout.Create(nil);
  try
    fmAbout.ShowModal();
  finally
    fmAbout.Free();
  end;
end;

procedure TfmMain.miExitClick(Sender: TObject);
begin
  Close();
end;

procedure TfmMain.FormCreate(Sender: TObject);
var
  I: integer;
  Item: TMenuItem;
begin
  FComboIndex := 0;
  Application.Title := sAppTitle;
  for I := -1 to fmMain.LangSource.Languages.Count - 1 do begin
    Item := TMenuItem.Create(Self);
    if I = -1
      then Item.Caption := fmMain.LangSource.OriginalName
      else Item.Caption := fmMain.LangSource.LanguageName[I];
    Item.Tag := I + 2;
    Item.RadioItem := True;
    Item.OnClick := LanguageItemClick;
    miLanguage.Insert(I + 1, Item);
    Item := TMenuItem.Create(Self);
    if I = -1
      then Item.Caption := fmMain.LangSource.OriginalName
      else Item.Caption := fmMain.LangSource.LanguageName[I];
    Item.Tag := I + 2;
    Item.RadioItem := True;
    Item.OnClick := LanguageItemClick;
    pmLanguages.Items.Insert(I + 1, Item);
  end;
  miLanguage.Items[fmMain.LangSource.ActiveLanguage + 1].Checked := True;
  pmLanguages.Items[fmMain.LangSource.ActiveLanguage + 1].Checked := True;
  cbSample.ItemIndex := 0;
end;

procedure TfmMain.LanguageItemClick(Sender: TObject);
var
  I: integer;
begin
  if Sender is TMenuItem then begin
    TMenuItem(Sender).Checked := True;
    for I := 0 to miLanguage.Count - 1 do
      if miLanguage.Items[I].Tag = TMenuItem(Sender).Tag then begin
        miLanguage.Items[I].Checked := True;
        Break;
      end;
    for I := 0 to pmLanguages.Items.Count - 1 do
      if pmLanguages.Items[I].Tag = TMenuItem(Sender).Tag then begin
        pmLanguages.Items[I].Checked := True;
        Break;
      end;
    if (TMenuItem(Sender).Tag > 0) and
       (TMenuItem(Sender).Tag < fmMain.LangSource.Languages.Count + 2)
      then fmMain.LangSource.ActiveLanguage := TMenuItem(Sender).Tag - 2;
  end;
end;

procedure TfmMain.pmLanguagesPopup(Sender: TObject);
begin
  if fmMain.LangSource.Languages.Count < pmLanguages.Items.Count
    then pmLanguages.Items[fmMain.LangSource.ActiveLanguage + 1].Checked := True;
end;

procedure TfmMain.LocalizerLanguageChanged(Sender: TObject);
begin
  inherited;
  if cbSample.Items.Count > FComboIndex
    then cbSample.ItemIndex := FComboIndex;
end;

procedure TfmMain.LangSourceLanguageChanged(Sender: TObject);
begin
  if LangSource.ActiveLanguage < 0 then
    Application.Title := sAppTitle
  else begin
    LangSource.FormSection := 'Additional Strings';
    Application.Title := LangSource.LoadString('sAppTitle', sAppTitle);
  end;
end;

procedure TfmMain.LocalizerLanguageChanging(Sender: TObject;
  LanguageIndex: Integer; var AllowChange: Boolean);
begin
  FComboIndex := cbSample.ItemIndex;
end;

procedure TfmMain.LangSourceLanguageOpen(Sender: TObject);
var
  FileName: string;
begin
  FStrings := TStringList.Create();
  FStrings.Sorted := True;
  FileName := LangSource.LanguageValue[LangSource.ActiveLanguage];
  if FileExists(FileName)
    then FStrings.LoadFromFile(FileName);
end;

procedure TfmMain.LangSourceLanguageClose(Sender: TObject);
begin
  FStrings.Free();
end;

procedure TfmMain.LangSourceClear(Sender: TObject);
var
  Index: integer;
begin
  if Trim(LangSource.FormSection) = '' then Exit;
  Index := 0;
  while CompareText(FStrings[Index], LangSource.FormSection) < 0 do begin
    Inc(Index);
    if Index > FStrings.Count then Exit;
  end;
  while Pos(LangSource.FormSection + '.', FStrings[Index]) = 1 do
    FStrings.Delete(Index);
  FStrings.SaveToFile(LangSource.LanguageValue[LangSource.ActiveLanguage]);
end;

procedure TfmMain.LangSourceClearAll(Sender: TObject);
begin
  FStrings.Clear();
  FStrings.SaveToFile(LangSource.LanguageValue[LangSource.ActiveLanguage]);
end;

procedure TfmMain.LangSourceLoadString(Sender: TObject;
  const StringName: String; var Value: String);
var
  Index: integer;
  S: string;
begin
  if Trim(LangSource.FormSection) = '' then Exit;
  S := LangSource.FormSection + '::' + StringName + '=';
  Index := 0;
  while (FStrings.Count > Index) and (CompareText(FStrings[Index], S) < 0) do
    Inc(Index);
  if (FStrings.Count > Index) and (Pos(S, FStrings[Index]) = 1) then
    Value := GetDecodedString(Copy(FStrings[Index], Length(S) + 1,
      Length(FStrings[Index]) - Length(S)), '"', '#');
end;

procedure TfmMain.LangSourceLoadStrings(Sender: TObject;
  Strings: TStrings);
var
  Index, I: integer;
  S, StringName: string;
  Value: string;
begin
  if Trim(LangSource.FormSection) = '' then Exit;
  Strings.Clear();
  S := LangSource.FormSection + '::';
  Index := 0;
  while (FStrings.Count > Index) and (CompareText(FStrings[Index], S) < 0) do
    Inc(Index);
  while (FStrings.Count > Index) and (Pos(S, FStrings[Index]) = 1) do begin
    I := Pos('=', FStrings[Index]);
    if I < Length(S) then Continue;
    StringName := Copy(FStrings[Index], Length(S) + 1, I - Length(S) - 1);
    Value := Copy(FStrings[Index], I + 1, Length(FStrings[Index]) - I);
    Strings.Values[StringName] := GetDecodedString(Value, '"', '#');
    Inc(Index);
  end;
end;

end.
